"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchFiles = exports.generate = void 0;
/*
 * @vuepress
 * ---
 * title: Main
 * headline: The Main File
 * ---
 */
const chokidar_1 = __importDefault(require("chokidar"));
const del_1 = __importDefault(require("del"));
const kleur_1 = __importDefault(require("kleur"));
const mkdirp_1 = require("mkdirp");
const promises_1 = __importDefault(require("node:fs/promises"));
const node_readline_1 = __importDefault(require("node:readline"));
const constants_1 = require("./constants");
const list_folder_1 = require("./lib/list-folder");
const parser_1 = require("./lib/parser");
const vue_sidebar_1 = require("./lib/vue-sidebar");
/**
 * Create the sidebar
 * @param {object} options
 * @returns {Promise}
 */
const createVuepressSidebar = (options) => promises_1.default.writeFile(`${options.docsFolder}/config.js`, `exports.fileTree=${JSON.stringify(options.fileTree)};exports.sidebarTree = (title = 'Mainpage') => (${JSON.stringify((0, vue_sidebar_1.generateVueSidebar)(options)).replace('::vuepress-jsdoc-title::', '"+title+"')});`);
/**
 * Parse file
 * @param {DirectoryFile} file
 * @param {CLIArguments} argv
 * @returns {Promise}
 */
const parseDirectoryFile = (file, argv) => {
    const { srcFolder, docsFolder, partials } = parseArguments(argv);
    if (!file.isDir && file.folder) {
        if (['.jsx', '.tsx', '.ts', '.js'].includes(file.ext || '')) {
            return (0, parser_1.parseFile)(file, srcFolder, docsFolder, argv.jsDocConfigPath, partials);
        }
        if (file.ext === '.vue') {
            return (0, parser_1.parseVueFile)(file, srcFolder, docsFolder);
        }
    }
};
/**
 * Create the readme file. First page when entering the documentation.
 * @param {CLIArguments} argv
 * @param {string} deletedPaths
 * @returns {void}
 */
const createReadmeFile = (argv, deletedPaths) => __awaiter(void 0, void 0, void 0, function* () {
    const { srcFolder, codeFolder, docsFolder, title, readme } = parseArguments(argv);
    let readMeContent = `### Welcome to ${title}`;
    const readmePath = readme || `${srcFolder}/README.md`;
    try {
        readMeContent = yield promises_1.default.readFile(readmePath, 'utf-8');
        if (deletedPaths === null || deletedPaths === void 0 ? void 0 : deletedPaths.some((p) => p.indexOf(`${codeFolder}/README.md`) !== -1)) {
            console.log(`\n${kleur_1.default.white().bgBlack(' README ')} ${kleur_1.default.dim(readmePath.replace('README.md', ''))}${kleur_1.default.bold('README.md')} \u2192  ${kleur_1.default.dim(docsFolder)}${kleur_1.default.bold('/README.md')}`);
        }
    }
    catch (e) {
        console.log(`\n${kleur_1.default.white().bgBlack(' README ')} Add default README.md`);
    }
    yield promises_1.default.writeFile(`${docsFolder}/README.md`, readMeContent);
});
/**
 * Parse all CLI arguments
 * @param {CLIArguments} argv
 * @returns {object} all arguments
 */
const parseArguments = (argv) => {
    return {
        include: (argv.include || '').split(',').filter(Boolean),
        exclude: (argv.exclude || '').split(',').filter(Boolean),
        srcFolder: argv.source.replace('./', ''),
        codeFolder: argv.folder,
        docsFolder: `${argv.dist}/${argv.folder}`,
        title: argv.title,
        readme: argv.readme,
        rmPattern: argv.rmPattern || [],
        partials: argv.partials || []
    };
};
/**
 * Default command that generate md files
 * @param {object} argv passed arguments
 */
const generate = (argv) => __awaiter(void 0, void 0, void 0, function* () {
    const { exclude, include, srcFolder, codeFolder, docsFolder, title, rmPattern } = parseArguments(argv);
    const startTime = +new Date();
    // remove docs folder, except README.md
    const deletedPaths = yield (0, del_1.default)([`${docsFolder}/**/*`, ...rmPattern]);
    const lsFolder = yield (0, list_folder_1.listFolder)(srcFolder, exclude, include);
    yield (0, mkdirp_1.mkdirp)(docsFolder);
    const parsePromises = [];
    // print out all files
    for (const file of lsFolder.paths) {
        if (!file.isDir) {
            node_readline_1.default.clearLine(process.stdout, 0);
            node_readline_1.default.cursorTo(process.stdout, 0);
            process.stdout.write(kleur_1.default.dim(` ${file.path} `));
            yield new Promise((resolve) => setTimeout(resolve, 20));
        }
    }
    node_readline_1.default.clearLine(process.stdout, 0);
    node_readline_1.default.cursorTo(process.stdout, 0);
    // iterate through files and parse them
    for (const file of lsFolder.paths) {
        parsePromises.push((() => __awaiter(void 0, void 0, void 0, function* () {
            const data = yield parseDirectoryFile(file, argv);
            if (data && (data === null || data === void 0 ? void 0 : data.relativePathDest)) {
                return (0, parser_1.writeContentToFile)(data, data.relativePathDest);
            }
        }))());
    }
    // wait unitl all files resolved
    let result = yield Promise.all(parsePromises);
    result = result.filter(Boolean);
    // write vuepress sidebar
    yield createVuepressSidebar({
        fileTree: lsFolder.tree,
        srcFolder,
        docsFolder,
        codeFolder,
        title
    });
    // print stats
    for (const file of lsFolder.excluded) {
        console.log(kleur_1.default.reset().inverse().bold().blue(' EXCLUDE '), `${kleur_1.default.dim(file.folder)}${kleur_1.default.bold(file.name + file.ext)}`);
    }
    console.log();
    for (const entry of result.flat()) {
        if (!entry.file)
            continue;
        const color = {
            [constants_1.StatisticType.INCLUDE]: kleur_1.default.reset().inverse().bold().green,
            [constants_1.StatisticType.ERROR]: kleur_1.default.reset().inverse().bold().red,
            [constants_1.StatisticType.EMPTY]: kleur_1.default.reset().inverse().bold().yellow,
            [constants_1.StatisticType.EXCLUDE]: kleur_1.default.reset().inverse().bold().blue
        }[entry.type];
        console.log(color(` ${entry.type} `), `${kleur_1.default.dim(entry.relativePathSrc)}${kleur_1.default.bold(entry.file.name + entry.file.ext)} \u2192 ${kleur_1.default.dim(entry.relativePathDest)}${kleur_1.default.bold(entry.file.name + '.md')}`);
    }
    // create README.md
    yield createReadmeFile(argv, deletedPaths);
    const resultTime = (Math.abs(startTime - +new Date()) / 1000).toFixed(2);
    console.log(`\n⏰ Time: ${resultTime}s`);
    (0, exports.watchFiles)(argv);
});
exports.generate = generate;
/**
 * Watch files in source folder
 * @param {CLIArguments} argv
 */
const watchFiles = (argv) => {
    const { exclude, include, srcFolder, codeFolder, docsFolder, title } = parseArguments(argv);
    if (argv.watch) {
        console.log('\n---\n\n👀 watching files...');
        const watcher = chokidar_1.default.watch([srcFolder, argv.readme, `${srcFolder}/README.md`].filter(Boolean), {
            ignored: /(^|[\/\\])\../,
            persistent: true
        });
        watcher.on('change', (path) => __awaiter(void 0, void 0, void 0, function* () {
            var _a;
            const lsFolder = yield (0, list_folder_1.listFolder)(srcFolder, exclude, include);
            const file = lsFolder.paths.find((p) => p.path === path);
            node_readline_1.default.clearLine(process.stdout, 0);
            node_readline_1.default.cursorTo(process.stdout, 0);
            if (path === 'README.md' || path === `${srcFolder}/README.md`) {
                yield createReadmeFile(argv);
            }
            if (file) {
                process.stdout.write(kleur_1.default.dim(`update ${file.name + file.ext}`));
                (_a = parseDirectoryFile(file, argv)) === null || _a === void 0 ? void 0 : _a.then((data) => {
                    if (data) {
                        (0, parser_1.writeContentToFile)(data, data.relativePathDest);
                    }
                    createVuepressSidebar({
                        fileTree: lsFolder.tree,
                        srcFolder,
                        docsFolder,
                        codeFolder,
                        title
                    });
                });
            }
        }));
    }
};
exports.watchFiles = watchFiles;
