"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeContentToFile = exports.parseVueFile = exports.parseFile = void 0;
/*
 * @vuepress
 * ---
 * headline: lib/parser.ts
 * ---
 */
const jsdoc_to_markdown_1 = __importDefault(require("jsdoc-to-markdown"));
const mkdirp_1 = require("mkdirp");
const promises_1 = __importDefault(require("node:fs/promises"));
const node_path_1 = require("node:path");
const compileTemplates_1 = __importDefault(require("vue-docgen-cli/lib/compileTemplates"));
const docgen_1 = require("vue-docgen-cli/lib/docgen");
const constants_1 = require("../constants");
const comment_parser_1 = require("./comment-parser");
/**
 * Parse a typescript or javascript file
 * @param file {DirectoryFile}
 * @param srcFolder {string}
 * @param destFolder {string}
 * @param configPath {string}
 * @param partials {string | string[]}
 * @returns {object} file data
 */
const parseFile = (file, srcFolder, destFolder, configPath, partials) => __awaiter(void 0, void 0, void 0, function* () {
    if (!file.folder)
        return null;
    const root = process.cwd();
    const relativePathDest = (0, node_path_1.join)(destFolder, file.folder.replace(srcFolder, ''));
    const folderInDest = (0, node_path_1.join)(root, relativePathDest);
    const folderInSrc = (0, node_path_1.join)(root, file.folder);
    let success = true;
    let empty = false;
    let excluded = false;
    let fileContent = '';
    // parse file
    try {
        let content = '';
        let fileName = file.name;
        if (fileName.endsWith('index')) {
            fileName = 'index';
        }
        content = yield jsdoc_to_markdown_1.default.render({
            'no-cache': configPath ? true : false,
            files: [(0, node_path_1.join)(process.cwd(), file.folder, fileName + file.ext)],
            configure: configPath,
            partial: [
                (0, node_path_1.resolve)(__filename, '../../../template/header.hbs'),
                (0, node_path_1.resolve)(__filename, '../../../template/main.hbs'),
                ...partials
            ]
        });
        fileContent = (0, comment_parser_1.parseVuepressFileHeader)(yield promises_1.default.readFile(`${(0, node_path_1.join)(folderInSrc, fileName + file.ext)}`, 'utf-8'), file);
        if (content) {
            fileContent += content;
        }
        else {
            empty = true;
        }
    }
    catch (e) {
        success = false;
        excluded = true;
    }
    return {
        success,
        file,
        empty,
        excluded,
        relativePathDest,
        relativePathSrc: file.folder,
        dest: folderInDest,
        content: fileContent
    };
});
exports.parseFile = parseFile;
/**
 * Parse a vue file
 * @param {DirectoryFile} file
 * @param {string} srcFolder
 * @param {string} destFolder
 * @returns {object} file data
 */
const parseVueFile = (file, srcFolder, destFolder) => __awaiter(void 0, void 0, void 0, function* () {
    if (!file.folder)
        return null;
    const root = process.cwd();
    const relativePathDest = (0, node_path_1.join)(destFolder, file.folder.replace(srcFolder, ''));
    const folderInDest = (0, node_path_1.join)(root, relativePathDest);
    const folderInSrc = (0, node_path_1.join)(root, file.folder);
    const vueDocGenCliConf = yield (0, docgen_1.extractConfig)((0, node_path_1.join)(root, file.folder));
    const config = Object.assign(Object.assign({}, vueDocGenCliConf), { components: file.name + file.ext });
    let success = true;
    let empty = false;
    let fileContent = '';
    try {
        let fileName = file.name;
        if (fileName.endsWith('index')) {
            fileName = 'index';
        }
        // parse file
        const data = yield (0, compileTemplates_1.default)('add', (0, node_path_1.join)(config.componentsRoot, fileName + file.ext), config, fileName + file.ext);
        fileContent = (0, comment_parser_1.parseVuepressFileHeader)(yield promises_1.default.readFile(`${(0, node_path_1.join)(folderInSrc, fileName + file.ext)}`, 'utf-8'), file);
        if (data.content) {
            fileContent += data.content;
        }
        else {
            empty = true;
        }
    }
    catch (e) {
        console.log(e);
        success = false;
    }
    return {
        success,
        file,
        empty,
        relativePathDest,
        relativePathSrc: file.folder,
        dest: folderInDest,
        content: fileContent
    };
});
exports.parseVueFile = parseVueFile;
/**
 * Write content on disk
 * @param {ParseReturn | null} parseData
 * @param {string} dest
 * @returns {Promise | null} null or type with some data of the saved file
 */
const writeContentToFile = (parseData, dest) => __awaiter(void 0, void 0, void 0, function* () {
    const root = process.cwd();
    dest = (0, node_path_1.join)(root, dest);
    let type = constants_1.StatisticType.ERROR;
    if (parseData === null || parseData === void 0 ? void 0 : parseData.excluded) {
        type = constants_1.StatisticType.EXCLUDE;
    }
    try {
        if (parseData === null || parseData === void 0 ? void 0 : parseData.content) {
            const path = `${(0, node_path_1.join)(dest, parseData.file.name)}.md`;
            yield (0, mkdirp_1.mkdirp)(dest);
            yield promises_1.default.writeFile(path, parseData.content, 'utf-8');
            type = (parseData === null || parseData === void 0 ? void 0 : parseData.empty) ? constants_1.StatisticType.EMPTY : constants_1.StatisticType.INCLUDE;
        }
        return Object.assign(Object.assign({}, parseData), { type });
    }
    catch (_a) { }
    return null;
});
exports.writeContentToFile = writeContentToFile;
